@@ -1,9 +1,11 @@
 package Sentry::Raven;
 
 use 5.008;
-use Moose;
+use strict;
+use Moo;
+use MooX::Types::MooseLike::Base qw/ ArrayRef HashRef Int Str /;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 use DateTime;
 use English '-no_match_vars';
@@ -20,7 +22,7 @@ Sentry::Raven - A perl sentry client
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =head1 SYNOPSIS
 
@@ -53,19 +55,19 @@ Do not wait longer than this number of seconds when attempting to send an event.
 
 has [qw/ post_url public_key secret_key /] => (
     is       => 'ro',
-    isa      => 'Str',
+    isa      => Str,
     required => 1,
 );
 
 has sentry_version => (
     is      => 'ro',
-    isa     => 'Int',
+    isa     => Int,
     default => 3,
 );
 
 has timeout => (
     is      => 'ro',
-    isa     => 'Int',
+    isa     => Int,
     default => 5,
 );
 
@@ -83,10 +85,16 @@ has ua_obj => (
 
 has valid_levels => (
     is      => 'ro',
-    isa     => 'ArrayRef[Str]',
+    isa     => ArrayRef[Str],
     default => sub { [qw/ fatal error warning info debug /] },
 );
 
+has options => (
+    is      => 'ro',
+    isa     => HashRef[],
+    default => sub { { } },
+);
+
 around BUILDARGS => sub {
     my ($orig, $class, %args) = @_;
 
@@ -114,12 +122,53 @@ around BUILDARGS => sub {
         post_url   => $post_url,
         public_key => $public_key,
         secret_key => $secret_key,
+        options    => \%args,
 
         (defined($args{timeout}) ? (timeout => $args{timeout}) : ()),
         (defined($args{ua_obj})  ? (ua_obj  => $args{ua_obj})  : ()),
     );
 };
 
+=head2 $raven->capture_errors( $subref, %options )
+
+Execute the $subref and report any exceptions (die) back to the sentry service.  This automatically includes a stacktrace.  This requires C<$SIG{__DIE__}> so be careful not to override it in subsequent code or error reporting will be impacted.
+
+=cut
+
+sub capture_errors {
+    my ($self, $subref, %options) = @_;
+
+    local $SIG{__DIE__} = sub {
+        my ($message) = @_;
+        chomp($message);
+
+        my @frames;
+        my $depth = 1;
+        while (my @frame = caller($depth++)) {
+            push @frames, {
+                module   => $frame[0],
+                filename => $frame[1],
+                lineno   => $frame[2],
+                function => $frame[3],
+            };
+        }
+        @frames = reverse @frames;
+
+        my $event = $self->_generate_event(
+            %options,
+            message => $message,
+            culprit => $PROGRAM_NAME,
+        );
+
+        $event = $self->_add_exception_to_event($event, 'Die', $message);
+        $event = $self->_add_stacktrace_to_event($event, \@frames);
+
+        $self->_post_event($event);
+    };
+
+    return $subref->();
+};
+
 
 =head2 $raven->capture_message( $message, %options )
 
@@ -129,12 +178,8 @@ Post a string message to the sentry service.  Returns the event id.
 
 sub capture_message {
     my ($self, $message, %options) = @_;
-    return $self->_post_event($self->_generate_message_event($message, %options));
-};
-
-sub _generate_message_event {
-    my ($self, $message, %options) = @_;
-    return $self->_generate_event(message => $message, %options);
+    my $event = $self->_generate_event(message => $message, %options);
+    return $self->_post_event($event);
 };
 
 =head2 $raven->capture_exception( $exception_type, $exception_value, %options )
@@ -145,13 +190,12 @@ Post an exception type and value to the sentry service.  Returns the event id.
 
 sub capture_exception {
     my ($self, $type, $value, %options) = @_;
-    return $self->_post_event($self->_generate_exception_event($type, $value, %options));
+    my $event = $self->_add_exception_to_event($self->_generate_event(%options), $type, $value);
+    return $self->_post_event($event);
 };
 
-sub _generate_exception_event {
-    my ($self, $type, $value, %options) = @_;
-
-    my $event = $self->_generate_event(%options);
+sub _add_exception_to_event {
+    my ($self, $event, $type, $value) = @_;
 
     $event->{'sentry.interfaces.Exception'} = {
         type  => $type,
@@ -161,6 +205,119 @@ sub _generate_exception_event {
     return $event;
 };
 
+=head2 $raven->capture_request( $url, %request_options, %options )
+
+Post a web url request to the sentry service.  Returns the event id.
+
+C<%options> can contain:
+
+=over
+
+=item I<method =E<gt> 'GET'>
+
+=item I<data =E<gt> { $key =E<gt> $value }>
+
+=item I<query_string =E<gt> 'foo=bar'>
+
+=item I<cookies =E<gt> 'foo=bar'>
+
+=item I<headers =E<gt> { 'Content-Type' =E<gt> 'text/html' }>
+
+=item I<C<env> =E<gt> { REMOTE_ADDR =E<gt> '192.168.0.1' }>
+
+=back
+
+=cut
+
+sub capture_request {
+    my ($self, $url, %options) = @_;
+    my $event = $self->_add_request_to_event($self->_generate_event(%options), $url, %options);
+    return $self->_post_event($event);
+};
+
+sub _add_request_to_event {
+    my ($self, $event, $url, %options) = @_;
+
+    $event->{'sentry.interfaces.Http'} = {
+        url          => $url,
+        method       => $options{method},
+        data         => $options{data},
+        query_string => $options{query_string},
+        cookies      => $options{cookies},
+        headers      => $options{headers},
+        env          => $options{env},
+    };
+
+    return $event;
+};
+
+=head2 $raven->capture_stacktrace( $frames, %options )
+
+Post a stacktrace to the sentry service.  Returns the event id.
+
+C<$frames> is an arrayref of hashrefs with each hashref representing a single frame.
+
+    my $frames = [
+        {
+            filename => 'my/file1.pl',
+            function => 'function1',
+            vars     => { foo => 'bar' },
+            lineno   => 10,
+        },
+        {
+            filename => 'my/file2.pl',
+            function => 'function2',
+            vars     => { bar => 'baz' },
+            lineno   => 20,
+        },
+    ];
+
+The first frame should be the oldest frame.  Frames must contain at least one of C<filename>, C<function>, or C<module>.  These additional attributes are also supported:
+
+=over
+
+=item I<filename =E<gt> $file_name>
+
+=item I<function =E<gt> $function_name>
+
+=item I<module =E<gt> $module_name>
+
+=item I<C<lineno> =E<gt> $line_number>
+
+=item I<C<colno> =E<gt> $column_number>
+
+=item I<abs_path =E<gt> $absolute_path_file_name>
+
+=item I<context_line =E<gt> $line_of_code>
+
+=item I<pre_context =E<gt> [ $previous_line1, $previous_line2 ]>
+
+=item I<post_context =E<gt> [ $next_line1, $next_line2 ]>
+
+=item I<in_app =E<gt> $one_if_not_external_library>
+
+=item I<vars =E<gt> { $variable_name =E<gt> $variable_value }>
+
+=back
+
+=cut
+
+sub capture_stacktrace {
+    my ($self, $frames, %options) = @_;
+    my $event = $self->_add_stacktrace_to_event($self->_generate_event(%options), $frames);
+    return $self->_post_event($event);
+};
+
+sub _add_stacktrace_to_event {
+    my ($self, $event, $frames) = @_;
+
+    $event->{'sentry.interfaces.Stacktrace'} = {
+        frames => $frames,
+    };
+
+    return $event;
+};
+
 sub _post_event {
     my ($self, $event) = @_;
 
@@ -199,21 +356,31 @@ sub _generate_event {
     my ($self, %options) = @_;
 
     return {
-        event_id    => $options{event_id}    || _generate_id(),
-        timestamp   => $options{timestamp}   || DateTime->now()->iso8601(),
-        logger      => $options{logger}      || 'root',
-        server_name => $options{server_name} || hostname(),
-        platform    => $options{platform}    || 'perl',
+        event_id    => $options{event_id}    || $self->options()->{event_id}    || _generate_id(),
+        timestamp   => $options{timestamp}   || $self->options()->{timestamp}   || DateTime->now()->iso8601(),
+        logger      => $options{logger}      || $self->options()->{logger}      || 'root',
+        server_name => $options{server_name} || $self->options()->{server_name} || hostname(),
+        platform    => $options{platform}    || $self->options()->{platform}    || 'perl',
 
-        message     => $options{message},
-        culprit     => $options{culprit},
-        extra       => $options{extra}       || {},
-        tags        => $options{tags}        || {},
+        message     => $options{message}     || $self->options()->{message},
+        culprit     => $options{culprit}     || $self->options()->{culprit},
 
-        level       => $self->_validate_level($options{level}) || 'error',
+        extra       => $self->_merge_hashrefs($self->options()->{extra}, $options{extra}),
+        tags        => $self->_merge_hashrefs($self->options()->{tags}, $options{tags}),
+
+        level       => $self->_validate_level($options{level}) || $self->options()->{level} || 'error',
     };
 }
 
+sub _merge_hashrefs {
+    my ($self, $hash1, $hash2) = @_;
+
+    return {
+        ($hash1 ? %{ $hash1 } : ()),
+        ($hash2 ? %{ $hash2 } : ()),
+    };
+};
+
 sub _validate_level {
     my ($self, $level) = @_;
 
@@ -249,7 +416,7 @@ sub _build_ua_obj { LWP::UserAgent->new() }
 
 =head1 STANDARD OPTIONS
 
-These options can be passed to all of the C<capture_*> methods.
+These options can be passed to all methods accepting %options.  Passing these options to the constructor overrides defaults.
 
 =over 4
 
@@ -271,7 +438,7 @@ Event level of an event.  Acceptable values are C<fatal>, C<error>, C<warning>,
 
 =item I<logger =E<gt> 'root'>
 
-The creator of an event.  Defaults to current user.
+The creator of an event.  Defaults to 'root'.
 
 =item I<platform =E<gt> 'perl'>
 
@@ -301,6 +468,12 @@ A default DSN to be used if sentry_dsn is not passed to c<new>.
 
 =back
 
+=head1 LICENSE
+
+Copyright (C) 2014 by Rentrak Corporation
+
+The full text of this license can be found in the LICENSE file included with this module.
+
 =cut
 
 1;
